@@ -21,14 +21,21 @@
 #include <QPainter>
 #include <QPrintDialog>
 #include <QPrinter>
+#include <QPropertyAnimation>
 #include <QPushButton>
+#include <QScreen>
+#include <QStyle>
 #include <QTimer>
 #include <QVBoxLayout>
 #include <QVariantAnimation>
 #include <QtMath>
 
 #ifdef XCB_FOUND
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QX11Info>
+#else
+#include <private/qtx11extras_p.h>
+#endif
 #include <xcb/xcb.h>
 #endif
 
@@ -145,24 +152,8 @@ void KSMainWindow::init()
     mAnnotateButton->setText(i18n("Annotate"));
     mAnnotateButton->setToolTip(i18n("Add annotation to the screenshot"));
     mAnnotateButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-    mAnnotateButton->setIcon(QIcon::fromTheme(QStringLiteral("document-edit")));
-    connect(mAnnotateButton, &QToolButton::clicked, this, [this] {
-        if (mAnnotatorActive) {
-            mKSWidget->hideAnnotator();
-            mAnnotateButton->setText(i18n("Annotate"));
-        } else {
-            mKSWidget->showAnnotator();
-            mAnnotateButton->setText(i18n("Annotation Done"));
-        }
-
-        mToolsButton->setEnabled(mAnnotatorActive);
-        mSendToButton->setEnabled(mAnnotatorActive);
-        mClipboardButton->setEnabled(mAnnotatorActive);
-        mSaveButton->setEnabled(mAnnotatorActive);
-
-        mAnnotatorActive = !mAnnotatorActive;
-    });
-
+    mAnnotateButton->setIcon(QIcon::fromTheme(QStringLiteral("edit-image"), QIcon::fromTheme(QStringLiteral("document-edit"))));
+    connect(mAnnotateButton, &QToolButton::clicked, this, &KSMainWindow::updateAnnotatorVisibility);
     mDialogButtonBox->addButton(mAnnotateButton, QDialogButtonBox::ActionRole);
 #endif
 
@@ -398,6 +389,10 @@ void KSMainWindow::setScreenshotAndShow(const QPixmap &pixmap, bool showAnnotato
         setWindowModified(true);
     } else {
         restoreWindowTitle();
+        // check if there is an screenshot already visible, if true, don't disable buttons
+        // this check has to be done after if (mPixmapExists), otherwise it
+        // might set a new empty screenshot, overwriting the previous one
+        mPixmapExists = mKSWidget->isScreenshotSet();
     }
 
 #ifdef KIMAGEANNOTATOR_FOUND
@@ -653,3 +648,48 @@ void KSMainWindow::keyPressEvent(QKeyEvent *event)
     }
     QDialog::keyPressEvent(event);
 }
+
+#ifdef KIMAGEANNOTATOR_FOUND
+void KSMainWindow::updateAnnotatorVisibility()
+{
+    static QSize oldSize = size();
+    QScreen *screen = QGuiApplication::primaryScreen();
+
+    // animation required for non-maximized resizing
+    QPropertyAnimation *resizeAnimation = new QPropertyAnimation(this, "size");
+    auto kdeGlobalsConfig = KConfigGroup(KSharedConfig::openConfig(), QStringLiteral("KDE"));
+    float animationDurationFactor = std::max(0.0, kdeGlobalsConfig.readEntry("AnimationDurationFactor", 1.0));
+
+    resizeAnimation->setDuration(2 * style()->styleHint(QStyle::SH_Widget_Animation_Duration, nullptr, this) * animationDurationFactor);
+
+    if (mAnnotatorActive) {
+        mKSWidget->hideAnnotator();
+        if (windowState() == Qt::WindowMaximized) {
+            showNormal();
+        } else {
+            resizeAnimation->setEndValue(oldSize);
+            resizeAnimation->start();
+        }
+        mAnnotateButton->setText(i18nc("@action:button", "Annotate"));
+    } else {
+        mKSWidget->showAnnotator();
+        const auto sizeWhenAnnotating = mKSWidget->sizeHintWhenAnnotating();
+
+        if (KWindowSystem::isPlatformWayland() || screen->availableSize().height() <= sizeWhenAnnotating.height()
+            || screen->availableSize().width() <= sizeWhenAnnotating.width()) {
+            showMaximized();
+        } else {
+            resizeAnimation->setEndValue(sizeWhenAnnotating);
+            resizeAnimation->start();
+        }
+        mAnnotateButton->setText(i18nc("@action:button", "Annotation Done"));
+    }
+
+    mToolsButton->setEnabled(mAnnotatorActive);
+    mSendToButton->setEnabled(mAnnotatorActive);
+    mClipboardButton->setEnabled(mAnnotatorActive);
+    mSaveButton->setEnabled(mAnnotatorActive);
+
+    mAnnotatorActive = !mAnnotatorActive;
+}
+#endif
